#cs =========================================================================================================
; Author:   Roman Majewski
; Date:     11.08.2015
; FileName: ExcelInNeueExcelDatei.au3
;
; Beschreibung:
;   Script zur automatischen bertragung zweier Daten-Spalten aus mehreren Excel Dateien in eine einzige
;   Excel Datei auf mehrere Tabellenbltter und definierte Zielspalten.
;   Durchsucht bestimmte Verzeichnisse auf vorhandene Source-Dateien und transferiert die Daten in
;   unterschiedliche Tabellen (Sheets) der Ziel-Datei. Verschiebt die Source-Dateien anschlieend in einen
;   Unterordner, welcher bei Bedarf erstellt wird.
;   Sind alle Dateien eines Verzeichnisses abgearbeitet, knnen die Source-Dateien entweder in einer Datei
;   komprimiert, oder im Originalzustand zurck verschoben werden. Source-Verzeichnisse werden nur
;   ebenfalls in einen temporren Ordner verschoben und erst am Ende des Scripts zurck verschoben.
;
;   Durch die temporren Verzeichnisse kann bei unerwartetem Abbruch des Scriptes an der letzten Position
;   weitergearbeitet werden.
;
; Fuktion:
;   - Startet das gewhlte Export-Programm, schliet Meldung(en) beim Programm-Start.
;   - Dateistruktur untersttzt derzeit
;       . ein Root-Verzeichnis welches die Source-Verzeichisse enthlt und einem Filterkriterium unterliegen
;         knnen.
;       . die Source-Verzeichisse knnen eine undefinierte Anzahl an Unterordnern enthalten, welche ebenfalls
;         einem Filterkriterium entsprechen mssen.
;       . in allen Unterordnern wird nach den Source-Dateien gesucht welche wiederum einem Filterkriterium
;         unterliegen.
;       . werden Dateien gefunden die den Filterkriterien entsprechen, wird ein temporrer Ordner, falls er
;         nicht bereits existiert, erstellt und die abgearbeiteten Dateien hinein verschoben.
;       . sind alle Dateien eines Verzeichnisses abgearbeitet werden die Source-Dateien, wie definiert,
;         zurck verschoben und das temporre Verzeichnis gelscht.
;
; Wichtiger Hinweis:
;   !!! Vor dem Durchlauf des Scriptes immer eine vollstndige Sicherung der Daten durchfhren !!!
;
; Notiz:
;   Das Script wurde unter Windows 7 getestet
#ce =========================================================================================================


#include <Array.au3>
#include <File.au3>
#include <FileConstants.au3>
#include <AutoItConstants.au3>
#include <Excel.au3>
#include <MsgBoxConstants.au3>

#cs
; $strDataRootDir				Root Verzeichnis in welchem alle Source-Verzeichnisse mit den Daten abgelegt
; 								sind.
; $strDataDirFilter				Filterkriterium fr Data-Dirs, zB: "*"
; $strDataSubDirFilter			Filterkriterium fr Sub-Dirs, zB: "*Messung*"
; $strSourceFileFilter			Filterkriterium fr Source-Dateien, zB: "*.xls"
; $strTargetFileTemplate		DateiVorlage, wird kopiert und umbenannt
; $strTargetFilePath			Speicherort der Zieldatei, als Name der Zieldatei wird immer der Name des
;                               DataDir zugewiesen
; $strTargetFileExtension		Dateierweiterung der Zieldatei, zB: "*.xlsx"
; $strDirExportDone				Verzeichnis in welches abgearbeitete Source-Dateien vorbergehend verschoben
; 								werden, es wird bei Bedarf automatisch erstellt.
; 								Zum Schluss werden alle Dateien entweder archiviert (empfohlen) oder ins
; 								original-Verzeichnis zurck verschoben.
; $strDirExportError			Verzeichnis in welches inkorrekte Source-Dateien verschoben werden,
; 								es wird bei Bedarf automatisch erstellt.
; $strLogFile					Pfad zur Log-Datei, ist keine Datei angegeben wird im Script-Verzeichnis eine
; 								erstellt.
; $intErrExit					Wenn bentigte Dateien oder Verzeichnisse fehlen wird das Script abgebrochen
; 								  0 = OK
; 								  1 = Error

; $intSourceSheet				SourceTabelle
; $strSourceCellsZugkraft
; $strSourceCellsMZeit
; $strTargetSheets				A B C aus Dateinamen
; $strTargetCells
#ce


Dim $strProgramFiles_x86, $strProgramFiles_x64
$strProgramFiles_x86 = "C:\Program Files (x86)"
$strProgramFiles_x64 = "C:\Program Files"

Dim $strDataRootDir, $strDataDir, $strDataSubDirFilter, $strDataSubDir, $strSourceFileFilter
Dim $strTargetFileTemplate, $strTargetFile
Dim $strDirExportDone, $strDirExportError, $strLogFile, $strLogFileDefault
Dim $intErrExit
Dim $aryDataDirPath, $strDataDirName, $aryDataSubDirs
Dim $objExcelApp, $objTargetWorkbook, $objSourceWorkbook
Dim $intTargetSheet, $intSourceSheet, $strSourceCellsZugkraft, $strSourceCellsMZeit ,$strTargetCellsZugkraft, $strTargetCellsMZeit

; alle Variablen mssen angegeben sein
$strDataRootDir = "C:\Temp\AutoIT\RootDir"
$strDataDir = "C:\Temp\AutoIT\RootDir\Test Objekt"
$strDataSubDirFilter = "*Messung*"
$strSourceFileFilter = "*.xls"
$strTargetFileTemplate = "C:\Temp\AutoIT\RootDir\Vorlage_Auswertung.xlsx"
$strTargetFile = "C:\Temp\AutoIT\RootDir\Test Objekt\Target Object.xlsx"
$strDirExportDone = "_transferiert"
$strDirExportError = "_fehler"
$strLogFile = @ScriptDir & "\" & @ScriptName & ".log"
$intErrExit = 0

; Festlegen der Zellen der Source-Dateien
$varSourceSheet = 1
$SourceCellsZugkraft = "G13:G3252"
$SourceCellsMZeit = "Z13:Z3252"


; Ist keine Log-Datei vorhanden, wird eine Log-Datei im Scriptverzeichnis erstellt
If Not FileExists($strLogFile) Then
	; Log-Datei existiert nicht
	ConsoleWrite("Log-Datei existiert nicht" & @CRLF)
	If Not _FileCreate($strLogFile) Then
		; erstellen der Log-Datei fehlgeschlagen
		ConsoleWrite("erstellen der Log-Datei fehlgeschlagen" & @CRLF)
		MsgBox(0, "Script Error", @ScriptName & " wird beendet, da die Log-Datei nicht gefunden/erstellt werden konnte.")
		Exit
	Else
		; Log-Datei wurde erstellt
		ConsoleWrite("Log-Datei wurde erstellt" & @CRLF)
		_FileWriteLog($strLogFile, StringFormat("%-45s", "LogFile erstellt") & " " & $strLogFile & @CRLF)
	EndIf
EndIf


; Scriptstart loggen
_FileWriteLog($strLogFile, StringFormat("%-45s", "Script gestartet") & " " & "Script gestartet ..." & @CRLF)


; aktuellen Verzeichnisnamen von strDataDir extrahieren
$aryDataDirPath = StringSplit($strDataDir, "\")
$strDataDirName = $aryDataDirPath[UBound($aryDataDirPath) - 1]
ConsoleWrite("strDataDirName: " & $strDataDirName & @CRLF)

; Source-Verzeichnisse durchsuchen und alle gefilterten Sub-Verzeichnisse sequentiell abarbeiten
$aryDataSubDirs = _FileListToArrayRec($strDataDir, $strDataSubDirFilter, 2, 0, 1, 2)
_ArrayDisplay($aryDataSubDirs, "aryDataSubDirs")

; das erste Sub-Verzeichnis festlegen
$strDataSubDir = $aryDataSubDirs[1]
ConsoleWrite("strDataSubDir: " & $strDataSubDir & @CRLF)

; Sub-Verzeichnis nach dem Filter entsprechenden Source-Dateien durchsuchen
Local $arySourceFiles, $strSubDirOK, $strSubDirFalse
$arySourceFiles = _FileListToArrayRec($strDataSubDir, $strSourceFileFilter, 1, 0, 1, 2)
_ArrayDisplay($arySourceFiles, "arySourceFiles")
$strSubDirOK = $strDataSubDir & "\" & $strDirExportDone
$strSubDirFalse = $strDataSubDir & "\" & $strDirExportError

; Wenn Source-Dateien gefunden wurden
If IsArray($arySourceFiles) Then

	; Nur wenn neue Dateien vorhanden sind wird das temporre Verzeichnis bei Bedarf erstellt
	_ChkDir($intErrExit, 2, $strSubDirOK, "", _
			StringFormat("%-45s", "strDirExportDone fehlt"), _
			StringFormat("%-45s", "strDirExportDone vorhanden"), _
			StringFormat("%-45s", "strDirExportDone nicht erstellt"), _
			StringFormat("%-45s", "strDirExportDone erstellt"))

	; Nur wenn neue Dateien vorhanden sind wird die Ziel-Datei bei Bedarf erstellt.
	ConsoleWrite("strTargetFile: " & $strTargetFile & @CRLF)
	If Not FileExists($strTargetFile) Then
		_ChkFile($intErrExit, 5, $strTargetFile, $strTargetFileTemplate, _
				 StringFormat("%-45s", "strTargetFileTemplate fehlt"), _
				 StringFormat("%-45s", "strTargetFileTemplate vorhanden"), _
				 StringFormat("%-45s", ""), _
				 StringFormat("%-45s", ""))

		If @error = 1 Then
			_ErrExit("Die Zieldatei, $strTargetFile, konnte nicht erstellt werden")
		EndIf
		ConsoleWrite("strTargetFile wurde kopiert" & $strTargetFile & @CRLF)
		_FileWriteLog($strLogFile, StringFormat("%-45s", "strTargetFile wurde kopiert") & " " & $strTargetFile & @CRLF)
	Else
		_FileWriteLog($strLogFile, StringFormat("%-45s", "strTargetFile vorhanden") & " " & $strTargetFile & @CRLF)
	EndIf

	; Nur wenn neue Dateien vorhanden sind, ffnen von Excel und der Ziel-Datei
	_OpenExcelFile($objExcelApp, $objTargetWorkbook, $strTargetFile, False, True, True)

	; Alle dem Filter entsprechenden Source-Dateien abarbeiten
	For $k = 1 To UBound($arySourceFiles) -1
		Local $strFileName, $aryPath, $aryFileName, $aryTmp, $aryStringParts

		; Dateinamen zerlegen und Dateierweiterung entfernen
		$aryPath = StringSplit($arySourceFiles[$k], "\")
		$aryTmp = StringSplit($aryPath[UBound($aryPath) - 1], ".")
		$strFileName = StringTrimRight($aryPath[UBound($aryPath) - 1], StringLen($aryTmp[UBound($aryTmp) - 1]) + 1)

		; Der Dateiname ist jetzt: zB: "Vorname Nachname A 33-32#1"
		$aryFileName = StringSplit($strFileName, " ")
		;_ArrayDisplay($aryFileName, "Korrekte Parts")

		; Bei Bedarf wird ein temporres Verzeichnis erstellt, wohin die abgearbeiteten Dateien verschoben werden
		If $aryFileName[0] <> 4 Or Not ($aryFileName[1] & " " & $aryFileName[2]) = $strDataDirName Or Not ($aryFileName[3] >= "A" And $aryFileName[3] <= "C") Then
			_ChkDir($intErrExit, 2, $strSubDirFalse, "", _
					StringFormat("%-45s", "$strDirExportError fehlt"), _
					StringFormat("%-45s", "$strDirExportError OK"), _
					StringFormat("%-45s", "$strDirExportError nicht erstellt"), _
					StringFormat("%-45s", "$strDirExportError erstellt"))

			_FileWriteLog($strLogFile, StringFormat("%-45s", "Datei ungltig/unbekannt") & " " & $arySourceFiles[$k] & @CRLF)
			FileMove($arySourceFiles[$k], $strDataSubDir & "\" & $strDirExportError & "\")
			_FileWriteLog($strLogFile, StringFormat("%-45s", "Datei verschoben nach") & " " & $strDataSubDir & "\" & $strDirExportError & @CRLF)
		EndIf


		; Festlegen der Tabelle im Ziel-Workbook
		Switch $aryFileName[3]
			Case "A"
				$varTargetSheet = 1
			Case "B"
				$varTargetSheet = 2
			Case "C"
				$varTargetSheet = 3
		EndSwitch


		; Zerlegen des letzten Teiles des Dateinamens zur Festlegung in welche Ziel-Zellen bertragen werden muss
		$aryStringParts = StringSplit($aryFileName[4], "#")
		Switch $aryStringParts[1]
			Case "32-33", "33-32"
				$TargetCellsZugkraft = _Excel_ColumnToLetter($aryStringParts[2] * 2 - 1) & "3"
				$TargetCellsMZeit =    _Excel_ColumnToLetter($aryStringParts[2] * 2) & "3"

			Case "33-34", "34-33"
				$TargetCellsZugkraft = _Excel_ColumnToLetter(1*13 + $aryStringParts[2] * 2 - 1) & "3"
				$TargetCellsMZeit =    _Excel_ColumnToLetter(1*13 + $aryStringParts[2] * 2) & "3"

			Case "34-35", "35-34"
				$TargetCellsZugkraft = _Excel_ColumnToLetter(2*13 + $aryStringParts[2] * 2 - 1) & "3"
				$TargetCellsMZeit =    _Excel_ColumnToLetter(2*13 + $aryStringParts[2] * 2) & "3"

			Case "35-36", "36-35"
				$TargetCellsZugkraft = _Excel_ColumnToLetter(3*13 + $aryStringParts[2] * 2 - 1) & "3"
				$TargetCellsMZeit =    _Excel_ColumnToLetter(3*13 + $aryStringParts[2] * 2) & "3"

			Case "42-43"
				$TargetCellsZugkraft = _Excel_ColumnToLetter(4*13 + $aryStringParts[2] * 2 - 1) & "3"
				$TargetCellsMZeit =    _Excel_ColumnToLetter(4*13 + $aryStringParts[2] * 2) & "3"

			Case "43-44"
				$TargetCellsZugkraft = _Excel_ColumnToLetter(5*13 + $aryStringParts[2] * 2 - 1) & "3"
				$TargetCellsMZeit =    _Excel_ColumnToLetter(5*13 + $aryStringParts[2] * 2) & "3"

			Case "44-45"
				$TargetCellsZugkraft = _Excel_ColumnToLetter(6*13 + $aryStringParts[2] * 2 - 1) & "3"
				$TargetCellsMZeit =    _Excel_ColumnToLetter(6*13 + $aryStringParts[2] * 2) & "3"

			Case "45-46"
				$TargetCellsZugkraft = _Excel_ColumnToLetter(7*13 + $aryStringParts[2] * 2 - 1) & "3"
				$TargetCellsMZeit =    _Excel_ColumnToLetter(7*13 + $aryStringParts[2] * 2) & "3"

			Case Else
				_FileWriteLog($strLogFile, StringFormat("%-45s", "Datei nicht erforderlich") & " " & $arySourceFiles[$k] & @CRLF)
				FileMove($arySourceFiles[$k], $strDataSubDir & "\" & $strDirExportDone & "\")
				_FileWriteLog($strLogFile, StringFormat("%-45s", "Datei verschoben nach") & " " & $strDataSubDir & "\" & $strDirExportDone & @CRLF)
		EndSwitch

		; Source-Datei ffnen
		_OpenExcelFile($objExcelApp, $objSourceWorkbook, $arySourceFiles[$k], False, True, True)

		; Daten transferieren
		_ExcelToExcelSummary($objExcelApp, $objSourceWorkbook, $objTargetWorkbook, $varSourceSheet, $SourceCellsZugkraft, $varTargetSheet, $TargetCellsZugkraft, True)
		_ExcelToExcelSummary($objExcelApp, $objSourceWorkbook, $objTargetWorkbook, $varSourceSheet, $SourceCellsMZeit, $varTargetSheet, $TargetCellsMZeit, False)

		; Source-Datei schlieen
		_CloseExcelFile($objExcelApp, $objSourceWorkbook, False, False)


		; Versuche die Ziel-Datei zu speichern, alle fehlgeschlagen                 <===================  Tests speichern
		;$objTargetWorkbook.Activate
		;$objTargetWorkbook.Save
		;_Excel_BookSaveAs($objTargetWorkbook, $strTargetFile, Default, True)
		;Sleep(1000)

		_FileWriteLog($strLogFile, StringFormat("%-45s", "Datei erledigt") & " " & $arySourceFiles[$k] & @CRLF)
		FileMove($arySourceFiles[$k], $strDataSubDir & "\" & $strDirExportDone & "\")
		_FileWriteLog($strLogFile, StringFormat("%-45s", "Datei verschoben nach") & " " & $strDataSubDir & "\" & $strDirExportDone & @CRLF)
	Next
EndIf

;_Excel_BookSave($objTargetWorkbook)
_CloseExcelFile($objExcelApp, $objTargetWorkbook, True, False)

MsgBox(0, "bertrag fertig!", "Der bertrag wurde abgeschlossen.")

Exit
; --- End Body ----------------------------------------------------------------------------------------------


; --- Functions ---------------------------------------------------------------------------------------------
Func _ChkDir( ByRef $intErrExit, $intNeededCreate, ByRef $strPath1, $strPath2 = "", $strErrMsg = "", $strOKMsg = "", $strCreateErrMsg = "", $strCreateOKMsg = "")
	; Func:      		_ChkDir
	; Beschreibung:		prft ob das Verzeichnis $strPath1 existiert, erstellt das Verzeichnis bei Bedarf oder ndert strPath1 = strPath2.
	;                   Schreibt in die Log Datei.
	; Parameter:  		strPath1:         zu prfendes Verzeichnis
	;					strPath2:         alternatives Verzeichnis
	;					strErrMsg:        Log Fehler-Meldung
	;					strOKMsg:         Log OK-Meldung
	;					strCreateErrMsg:  Log Fehler-Meldung Erstellung Verzeichnis
	;					strCreateOKMsg:   Log OK-Meldung Erstellung Verzeichnis
	;					intErrExit:       0 = kein Fehler, 1 = Error Script muss nach verlassen der Funktion beendet werden
	;					intNeededCreate:  0 = Verzeichnis nicht erforderlich
	;					                  1 = erforderlich, wenn Verzeichnis strPath1 fehlt ErrorExit
	;					                  2 = erforderlich, bei Bedarf Verzeichnis auf strPath1 erstellen
	;					                  3 = erforderlich, bei Bedarf versuchen Verzeichnis auf strPath1 zu erstellen, strPath2 alternativ verwenden
	;					                  4 = erforderlich, bei Bedarf Verzeichnis nicht auf strPath1 erstellen, strPath2 alternativ verwenden
	;					                  5 = erforderlich, bei Bedarf Verzeichnis von strPath2 nach strPath1 kopieren
	; Rueckgabewert:	0 - OK; 1 - Fehler

	Local $intFileExist = FileExists($strPath1)

	Switch $intFileExist
		Case 1
			; Verzeichnis vorhanden
			ConsoleWrite("Verzeichnis vorhanden" & $strPath1 & @CRLF)
			_FileWriteLog($strLogFile, $strOKMsg  & " " & $strPath1 & @CRLF)
			Return 0
		Case 0
			; Verzeichnis nicht vorhanden oder Pfad Fehlerhaft
			ConsoleWrite("Verzeichnis nicht vorhanden oder Pfad Fehlerhaft" & $strPath1 & @CRLF)
			Switch $intNeededCreate
				Case 0
					; Verzeichnis nicht erforderlich
					ConsoleWrite("Verzeichnis ist nicht erforderlich" & $strPath1  & @CRLF)
					_FileWriteLog($strLogFile, $strErrMsg  & " " & $strPath1 & @CRLF)
					Return 0
				Case 1
					; erforderlich, wenn Verzeichnis strPath1 fehlt ErrorExit
					ConsoleWrite("Verzeichnis erforderlich, ErrorExit" & $strPath1 & @CRLF)
					_FileWriteLog($strLogFile, $strErrMsg  & " " & $strPath1 & @CRLF)
					$intErrExit = 1
					SetError(1)
					Return 1
				Case 2
					; erforderlich, bei Bedarf Verzeichnis auf strPath1 erstellen
					ConsoleWrite("Verzeichnis ist erforderlich, erstellen versuchen" & $strPath1 & @CRLF)
					_FileWriteLog($strLogFile, $strErrMsg  & " " & $strPath1 & @CRLF)
					If DirCreate($strPath1) Then
						; Verzeichnis wurde erstellt
						ConsoleWrite("Verzeichnis wurde erstellt" & $strPath1 & @CRLF)
						_FileWriteLog($strLogFile, $strCreateOKMsg  & " " & $strPath1 & @CRLF)
						Return 0
					Else
						; Verzeichnis konnte nicht erstellt werden
						ConsoleWrite("Verzeichnis konnte nicht erstellt werden" & $strPath1 & @CRLF)
						_FileWriteLog($strLogFile, $strCreateErrMsg  & " " & $strPath1 & @CRLF)
						$intErrExit = 1
						SetError(1)
						Return 1
					EndIf
				Case 3
					; erforderlich, bei Bedarf versuchen Verzeichnis auf strPath1 zu erstellen, strPath2 alternativ verwenden
					ConsoleWrite("Verzeichnis ist erforderlich, wenn erstellen nicht mglich strPath2 verwenden" & $strPath1 & @CRLF)
					_FileWriteLog($strLogFile, $strErrMsg  & " " & $strPath1 & @CRLF)
					If DirCreate($strPath1) Then
						; Verzeichnis wurde erstellt
						ConsoleWrite("Verzeichnis wurde erstellt" & $strPath1 & @CRLF)
						_FileWriteLog($strLogFile, $strCreateOKMsg  & " " & $strPath1 & @CRLF)
						Return 0
					Else
						; Verzeichnis konnte nicht erstellt werden
						ConsoleWrite("Verzeichnis konnte nicht erstellt werden" & $strPath1 & @CRLF)
						_FileWriteLog($strLogFile, $strCreateErrMsg  & " " & $strPath1 & @CRLF)
						ContinueCase
					EndIf
				Case 4
					; erforderlich, bei Bedarf Verzeichnis nicht auf strPath1 erstellen, strPath2 alternativ verwenden
					ConsoleWrite("Verzeichnis erforderlich, alternativen Pfad verwenden" & $strPath1 & @CRLF)
					If $intFileExist = 4 Then
						_FileWriteLog($strLogFile, $strErrMsg  & " " & $strPath1 & @CRLF)
					EndIf
					If $strPath2 = "" Or $strPath2 = Null Or Not FileExists($strPath2) Then
						_FileWriteLog($strLogFile, StringFormat("%-45s", "Altern. Verzeichnis Error")  & " " & $strPath1 & @CRLF)
						$intErrExit = 1
						SetError(1)
						Return 1
					Else
						$strPath1 = $strPath2
						_FileWriteLog($strLogFile, StringFormat("%-45s", "Altern. Verzeichnis verwenden")  & " " & $strPath1 & @CRLF)
						Return 0
					EndIf
				Case 5
					; erforderlich, bei Bedarf Verzeichnis von strPath2 nach strPath1 kopieren
					_FileWriteLog($strLogFile, $strErrMsg  & " " & $strPath1 & @CRLF)
					If $strPath2 = "" Or $strPath2 = Null Or Not FileExists($strPath2) Then
						_FileWriteLog($strLogFile, StringFormat("%-45s", "Altern. Verzeichnis Error")  & " " & $strPath1 & @CRLF)
						$intErrExit = 1
						SetError(1)
						Return 1
					Else
						FileCopy($strPath2, $strPath1, 0)
						_FileWriteLog($strLogFile, StringFormat("%-45s", "Altern. Verzeichnis kopiert")  & " " & $strPath1 & @CRLF)
						Return 0
					EndIf
				Case Else
					; Ein unbekannter Fehler ist aufgetreten
					ConsoleWrite("Ein unbekannter Fehler ist aufgetreten" & $strPath1 & @CRLF)
					_FileWriteLog($strLogFile, StringFormat("%-45s", "Unbekannter Fehler")  & " " & "intNeededCreate Case-Nummer:" & $intNeededCreate & ", " & $strPath1 & @CRLF)
					$intErrExit = 1
					SetError(1)
					Return 1
			EndSwitch
		Case Else
			; Ein unbekannter Fehler ist aufgetreten
			ConsoleWrite("Ein unbekannter Fehler ist aufgetreten" & $strPath1 & @CRLF)
			_FileWriteLog($strLogFile, StringFormat("%-45s", "Unbekannter Fehler")  & " " & "aryFileList @error:" & @error & ", " & $strPath1 & @CRLF)
			$intErrExit = 1
			SetError(1)
			Return 1
	EndSwitch
EndFunc ; ==> _ChkDir


Func _ChkFile(ByRef $intErrExit, $intNeededCreate, ByRef $strPath1, $strPath2 = "", $strErrMsg = "", $strOKMsg = "", $strCreateErrMsg = "", $strCreateOKMsg = "")
	; Func:      		_ChkFile
	; Beschreibung:		prft ob die Datei $strPath1 existiert, erstellt die Datei bei Bedarf oder ndert strPath1 = strPath2.
	;                   Schreibt in die Log Datei.
	; Parameter:  		strPath1:         zu prfende Datei
	;					strPath2:         alternative Datei
	;					strErrMsg:        Log Fehler-Meldung
	;					strOKMsg:         Log OK-Meldung
	;					strCreateErrMsg:  Log Fehler-Meldung Erstellung Datei
	;					strCreateOKMsg:   Log OK-Meldung Erstellung Datei
	;					intErrExit:       0 = kein Fehler, 1 = Error Script wird beendet
	;					intNeededCreate:  0 = Datei nicht erforderlich
	;					                  1 = erforderlich, wenn Datei strPath1 fehlt ErrorExit
	;					                  2 = erforderlich, bei Bedarf Datei auf strPath1 erstellen
	;					                  3 = erforderlich, bei Bedarf versuchen Datei auf strPath1 zu erstellen, strPath2 alternativ verwenden
	;					                  4 = erforderlich, bei Bedarf Datei nicht auf strPath1 erstellen, strPath2 alternativ verwenden
	;					                  5 = erforderlich, bei Bedarf Datei von strPath2 nach strPath1 kopieren
	; Rueckgabewert:	0 - OK; 1 - Fehler

	Local $intFileExist = FileExists($strPath1)

	Switch $intFileExist
		Case 1
			; Datei vorhanden
			ConsoleWrite("Datei vorhanden" & $strPath1 & @CRLF)
			_FileWriteLog($strLogFile, $strOKMsg  & " " & $strPath1 & @CRLF)
			Return 0
		Case 0
			; Datei nicht vorhanden
			ConsoleWrite("Datei nicht vorhanden" & $strPath1 & @CRLF)
			Switch $intNeededCreate
				Case 0
					; Datei ist nicht erforderlich
					ConsoleWrite("Datei nicht erforderlich" & $strPath1 & @CRLF)
					_FileWriteLog($strPath1, $strErrMsg  & " " & $strPath1 & @CRLF)
					Return 0
				Case 1
					; erforderlich, wenn Datei strPath1 fehlt ErrorExit
					ConsoleWrite("Datei erforderlich, ErrorExit" & $strPath1 & @CRLF)
					_FileWriteLog($strLogFile, $strErrMsg  & " " & $strPath1 & @CRLF)
					$intErrExit = 1
					SetError(1)
					Return 1
				Case 2
					; erforderlich, bei Bedarf Datei auf strPath1 erstellen
					_FileWriteLog($strLogFile, $strErrMsg  & " " & $strPath1 & @CRLF)
					ConsoleWrite("Datei ist erforderlich, erstellen versuchen" & $strPath1 & @CRLF)
					If _FileCreate($strPath1) Then
						; Datei wurde erstellt
						ConsoleWrite("Datei wurde erstellt" & $strPath1 & @CRLF)
						_FileWriteLog($strLogFile, $strCreateOKMsg  & " " & $strPath1 & @CRLF)
						Return 0
					Else
						; Datei konnte nicht erstellt werden
						ConsoleWrite("Datei konnte nicht erstellt werden" & $strPath1 & @CRLF)
						_FileWriteLog($strLogFile, $strCreateErrMsg  & " " & $strPath1 & @CRLF)
						$intErrExit = 1
						SetError(1)
						Return 1
					EndIf
				Case 3
					; erforderlich, bei Bedarf versuchen Datei auf strPath1 zu erstellen, strPath2 alternativ verwenden
					_FileWriteLog($strLogFile, $strErrMsg  & " " & $strPath1 & @CRLF)
					ConsoleWrite("Datei ist erforderlich, wenn erstellen nicht mglich strPath2 verwenden" & $strPath1 & @CRLF)
					If _FileCreate($strPath1) Then
						; Datei wurde erstellt
						ConsoleWrite("Datei wurde erstellt" & $strPath1 & @CRLF)
						_FileWriteLog($strLogFile, $strCreateOKMsg  & " " & $strPath1 & @CRLF)
						Return 0
					Else
						; Datei konnte nicht erstellt werden
						ConsoleWrite("Datei konnte nicht erstellt werden" & $strPath1 & @CRLF)
						_FileWriteLog($strLogFile, $strCreateErrMsg  & " " & $strPath1 & @CRLF)
						ContinueCase
					EndIf
				Case 4
					; erforderlich, bei Bedarf Datei nicht auf strPath1 erstellen, strPath2 alternativ verwenden
					If $intFileExist = 4 Then
						_FileWriteLog($strLogFile, $strErrMsg  & " " & $strPath1 & @CRLF)
					EndIf
					ConsoleWrite("Datei erforderlich, alternative Datei verwenden" & $strPath1 & @CRLF)
					If $strPath2 = "" Or $strPath2 = Null Or Not FileExists($strPath2) Then
						_FileWriteLog($strLogFile, StringFormat("%-45s", "Altern. Datei Error")  & " " & $strPath1 & @CRLF)
						$intErrExit = 1
						SetError(1)
						Return 1
					Else
						$strPath1 = $strPath2
						_FileWriteLog($strLogFile, StringFormat("%-45s", "Altern. Datei verwenden")  & " " & $strPath1 & @CRLF)
						Return 0
					EndIf
				Case 5
					; erforderlich, bei Bedarf Datei von strPath2 nach strPath1 kopieren
					_FileWriteLog($strLogFile, $strErrMsg  & " " & $strPath1 & @CRLF)
					If $strPath2 = "" Or $strPath2 = Null Or Not FileExists($strPath2) Then
						_FileWriteLog($strLogFile, StringFormat("%-45s", "Altern. Datei Error")  & " " & $strPath1 & @CRLF)
						$intErrExit = 1
						SetError(1)
						Return 1
					Else
						FileCopy($strPath2, $strPath1, 0)
						_FileWriteLog($strLogFile, StringFormat("%-45s", "Altern. Datei kopiert")  & " " & $strPath1 & @CRLF)
						Return 0
					EndIf
				Case Else
					; Ein unbekannter Fehler ist aufgetreten
					ConsoleWrite("Ein unbekannter Fehler ist aufgetreten" & $strPath1 & @CRLF)
					_FileWriteLog($strLogFile, StringFormat("%-45s", "Unbekannter Fehler")  & " " & "intNeededCreate Case-Nummer:" & $intNeededCreate & ", " & $strPath1 & @CRLF)
					$intErrExit = 1
					SetError(1)
					Return 1
			EndSwitch
		Case Else
			; Ein unbekannter Fehler ist aufgetreten
			ConsoleWrite("Ein unbekannter Fehler ist aufgetreten" & $strPath1 & @CRLF)
			_FileWriteLog($strLogFile, StringFormat("%-45s", "Unbekannter Fehler")  & " " & "aryFileList @error:" & @error & ", " & $strPath1 & @CRLF)
			$intErrExit = 1
			SetError(1)
			Return 1
	EndSwitch
EndFunc ; ==> _ChkFile


Func _OpenExcelFile(ByRef $objExcel, ByRef $objWorkbook, $strWorkbook, $blnReadOnly = False, $blnVisible = True, $blnInteractive = True)
	Local $sO1 = "No Object", $sO2 = "No Object", $sP1 = "No Workbook"
	If IsObj($objExcel) Then $sO1 = "Is Object"
	If IsObj($objWorkbook) Then $sO2 = "Is Object"
	If IsObj($strWorkbook) Then $sP1 = "Is Workbook"
	_FileWriteLog($strLogFile, StringFormat("%-45s", "_OpenExcelFile with") & " " & "objExcel: " & $sO1 & ", objWorkbook; " & $sO2 & ", strWorkbook; " & $strWorkbook & @CRLF)
	ConsoleWrite("_OpenExcelFile with; " & " " & "objExcel: " & $sO1 & ", objWorkbook; " & $sO2 & ", strWorkbook; " & $strWorkbook & @CRLF)

	If Not IsObj($objExcel) Then
		$objExcel = _Excel_Open() ;Default, Default, Default, $blnInteractive)
		If @error Then
			_FileWriteLog($strLogFile, StringFormat("%-45s", "_OpenExcelFile") & " " & "Error bei Erstellung des ExcelApplicationObject" & @CRLF)
			;MsgBox($MB_SYSTEMMODAL, "Excel UDF: _OpenExcelFile", "Error bei Erstellung des Excel Application Object." & @CRLF & "@error = " & @error & ", @extended = " & @extended)
			_ErrExit("")
		EndIf
	EndIf

	If Not IsObj($objWorkbook) Then
		$objWorkbook = _Excel_BookOpen($objExcel, $strWorkbook, $blnReadOnly, $blnVisible)
		If @error Then
			_FileWriteLog($strLogFile, StringFormat("%-45s", "_OpenExcelFile") & " " & "Error ffnen workbook: " & $objWorkbook & @CRLF)
			MsgBox($MB_SYSTEMMODAL, "Excel UDF: _OpenExcelFile", "Error ffnen workbook '" & $objWorkbook & "'." & @CRLF & "@error = " & @error & ", @extended = " & @extended)
			$objExcel.ScreenUpdating = True
			$objExcel.Interactive = True
			_Excel_Close($objExcel)
			_ErrExit("Schwerer Fehler")
		EndIf
	EndIf
EndFunc ; ==> _OpenExcelFile


Func _CloseExcelFile(ByRef $objExcel, ByRef $objWorkbook, $blnSaveChanges = True, $blnCloseApp = True)
	If IsObj($objWorkbook) Then
		$objExcel.ScreenUpdating = True
		$objExcel.Interactive = True
		If $blnSaveChanges = True Then
			$objExcel.Windows($objWorkbook.Name).Visible = True
		EndIf
		_Excel_BookClose($objWorkbook, $blnSaveChanges)
		If @error Then
			_FileWriteLog($strLogFile, StringFormat("%-45s", "_CloseExcelFile") & " " & "Error beim speichern von workbook " & @CRLF)
			MsgBox($MB_SYSTEMMODAL, "Excel UDF: _CloseExcelFile", "Error speichern workbook." & @CRLF & "@error = " & @error & ", @extended = " & @extended)
			_ErrExit("")
		EndIf
		sleep(300)
		If IsObj($objWorkbook) Then
			_FileWriteLog($strLogFile, StringFormat("%-45s", "_CloseExcelFile") & " " & "workbook existiert noch immer" & @CRLF)
			_Excel_BookClose($objWorkbook, $blnSaveChanges)
		EndIf
	EndIf
	If $blnCloseApp = True Then
		$objExcel.SaveChanges = True
		$objExcel.Interactive = True
		;msgbox(0, "", "Sollte Excel schlieen")
		_Excel_Close($objExcel, $blnSaveChanges)
	EndIf
EndFunc ; ==> _CloseExcelFile


Func _ExcelToExcelSummary(ByRef $objExcel, ByRef $objWorkbookS, ByRef $objWorkbookT, $SourceSheet, $SourceCells, $TargetSheet, $TargetCells, $blnMultiplicate)
	$objExcel.ScreenUpdating = False

	; Kopieren von Source-Datei
	Local $aryResult = _Excel_RangeRead($objWorkbookS, $SourceSheet, $SourceCells, 1)
	If @error Then
		_FileWriteLog($strLogFile, StringFormat("%-45s", "Excel UDF: _Excel_RangeRead") & "Fehler beim Lesen von der Source-Datei" & @CRLF)
		MsgBox($MB_SYSTEMMODAL, "Excel UDF: _Excel_RangeRead", "Error reading from workbook." & @CRLF & "@error = " & @error & ", @extended = " & @extended)
		_ErrExit("")
	EndIf
	;_ArrayDisplay($aryResult, "Excel UDF: _Excel_RangeRead - Cells " & $SourceCells & " of sheet " & $SourceSheet)


	; In Spalte Zugkraft mssen alle Werte multipliziert werden
	; Das Runden auf 2 Kommastellen erledigt Excel in der Standardformatierung beim Einfgen
	If $blnMultiplicate = True Then
		For $i = 0 To UBound($aryResult) -1
			$aryResult[$i] = $aryResult[$i] * 100
		Next
	EndIf
	;_ArrayDisplay($aryResult, "Excel UDF: _Excel_RangeRead - Cells *100")


	; Jede Zelle muss Wete enthalten, wegen Verkrzung auf 3 Sekunden Messzeit mssen leere Zellen am Ende entfernt werden
	If $aryResult[3200] = "" Then
		$index = 0
		For $j = 0 To UBound($aryResult) - 1
			If $aryResult[$j] <> "" Then
				$aryResult[$index] = $aryResult[$j]
				$Index += 1
			EndIf
		Next
		ReDim $aryResult[$index]
	EndIf


	; Einfgen in Ziel-Datei
	_Excel_RangeWrite($objWorkbookT, $TargetSheet, $aryResult, $TargetCells)
	If @error Then
		_FileWriteLog($strLogFile, StringFormat("%-45s", "Excel UDF: _Excel_RangeWrite") & " " & "Fehler beim Schreiben in die Ziel-Datei" & @CRLF)
		MsgBox($MB_SYSTEMMODAL, "Excel UDF: _Excel_RangeWrite", "Error writing to worksheet." & @CRLF & "@error = " & @error & ", @extended = " & @extended)
		_ErrExit("")
	EndIf
	$objExcel.ScreenUpdating = True
EndFunc ; ==> _ExcelToExcelSummary


Func _ErrExit($strErrMsg = "... Skript abgebrochen")
	_FileWriteLog($strLogFile, StringFormat("%-45s", "ERROR Exit") & " " & $strErrMsg & @CRLF)
	MsgBox(0, "AutoIt - Error", $strErrMsg & @CRLF)
	$objExcelApp.SaveChanges = True
	$objExcelApp.Interactive = True
	$objExcelApp.Quit
	Exit
EndFunc ; ==> _ErrExit
